import $ from 'jquery'

const Interactive = (($) => {
  /**
   * ------------------------------------------------------------------------
   * Constants
   * ------------------------------------------------------------------------
   */

  // const NAME = 'interactive'
  // const VERSION = '4.0.0'
  const DATA_KEY = 'tdu.interactive'
  const EVENT_KEY = `.${DATA_KEY}`

  const Events = {
    TAP: `tap${EVENT_KEY}`,
    PRESS: `press${EVENT_KEY}`,

    ACTIVE: 'mousedown touchstart',
    DEACTIVE: 'mouseup mouseout touchend touchcancel'
  }

  const ClassName = {
    FOCUS: 'focus',
    INTERACTIVE: 'interactive'
  }

  const Selector = {
    INTERACTIVE: '.interactive',
    FOCUS: '.click'
  }

  const Interval = {
    MIN_TRANSITION: 150,
    LONG_PRESS: 1000
  }

  /**
   * ------------------------------------------------------------------------
   * Api Definition
   * ------------------------------------------------------------------------
   */

  $(document).on(Events.ACTIVE, Selector.INTERACTIVE, function (event) {
    const startTime = Date.now()

    this.classList.add(ClassName.FOCUS)
    if (this.$previousHandler) {
      clearTimeout(this.$previousHandler)
      this.$previousHandler = 0
    }

    const releaseHandler = () => {
      // Clear long press timer
      this.$longPressHandler = 0
      this.$longPressTriggered = true

      const delta = Date.now() - startTime
      if (delta < Interval.MIN_TRANSITION) {
        // If delta is too short, delay transition.
        this.$previousHandler = setTimeout(() => {
          this.classList.remove(ClassName.FOCUS)
          $(this).off(Events.DEACTIVE, releaseHandler)
          this.$previousHandler = 0
        }, Interval.MIN_TRANSITION - delta)
      } else {
        // If delta is enough, delay
        this.classList.remove(ClassName.FOCUS)
        $(this).off(Events.DEACTIVE, releaseHandler)
        if (delta >= Interval.LONG_PRESS) {
          event.preventDefault()
          event.stopPropagation()
        }
      }
    }

    $(this).trigger($.Event(Events.TAP))

    // Add release mouse/touch handler
    $(this).on(Events.DEACTIVE, releaseHandler)

    // Add long-press handler
    this.$longPressTriggered = true
    this.$longPressHandler = setTimeout(() => {
      $(this).trigger($.Event(Events.PRESS))
      this.$longPressHandler = 0
    }, Interval.LONG_PRESS)
  })
})($)

export default Interactive
